"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MedScanSystem = void 0;
const SystemStatus_1 = require("./SystemStatus");
const events_1 = require("../events");
/**
 * Represents a system responsible for handling the medbay scan queue.
 *
 * See {@link MedScanSystemEvents} for events to listen to.
 */
class MedScanSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        super(ship, systemType, data);
        this.queue || (this.queue = []);
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn) {
        const numPlayers = reader.upacked();
        this.queue = [];
        for (let i = 0; i < numPlayers; i++) {
            const player = this.ship.room.getPlayerByPlayerId(reader.uint8());
            if (player)
                this.queue.push(player);
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn) {
        writer.upacked(this.queue.length);
        for (let i = 0; i < this.queue.length; i++) {
            if (this.queue[i].playerId)
                writer.uint8(this.queue[i].playerId);
        }
    }
    _removeFromQueue(player) {
        const idx = this.queue.indexOf(player);
        if (~idx) {
            this.queue.splice(idx, 1);
        }
    }
    async _joinQueue(player, rpc) {
        if (this.queue.includes(player))
            return;
        this._removeFromQueue(player);
        this.queue.push(player);
        this.dirty = true;
        const ev = await this.emit(new events_1.MedScanJoinQueueEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this._removeFromQueue(player);
        }
    }
    /**
     * Add a player to the queue.
     * @param player The player to add.
     */
    async addToQueue(player) {
        if (player.playerId === undefined)
            return;
        if (this.room.hostIsMe) {
            await this._joinQueue(player, undefined);
        }
        else {
            await this._sendRepair(player.playerId | 0x80);
        }
    }
    /**
     * Join the queue as the client's player.
     */
    async joinQueue() {
        if (!this.room.myPlayer)
            return;
        await this.addToQueue(this.room.myPlayer);
    }
    async _leaveQueue(player, rpc) {
        if (!this.queue.includes(player))
            return;
        this._removeFromQueue(player);
        this.dirty = true;
        const ev = await this.emit(new events_1.MedScanLeaveQueueEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.queue.push(player);
        }
    }
    /**
     * Remove a player from the queue.
     * @param player The player to remove.
     */
    async removeFromQueue(player) {
        if (player.playerId === undefined)
            return;
        if (this.room.hostIsMe) {
            await this._leaveQueue(player, undefined);
        }
        else {
            await this._sendRepair(player.playerId | 0x40);
        }
    }
    /**
     * Leave the queue as the current player.
     */
    async leaveQueue() {
        if (!this.room.myPlayer)
            return;
        await this.removeFromQueue(this.room.myPlayer);
    }
    async HandleRepair(player, amount, rpc) {
        const playerId = amount & 0x1f;
        const resolved = this.ship.room.getPlayerByPlayerId(playerId);
        if (resolved) {
            if (amount & 0x80) {
                await this._joinQueue(resolved, rpc);
            }
            else if (amount & 0x40) {
                await this._leaveQueue(resolved, rpc);
            }
        }
    }
}
exports.MedScanSystem = MedScanSystem;
//# sourceMappingURL=MedScanSystem.js.map